function [Y] = care_nwt_fac(Y0,A,B,C,tol,maxsteps)
%CARE_NWT_FAC
%
%  Newton's method for continuous-time algebraic Riccati equations
%
%  (CARE)  0  =  C'*C  +  A' X  +  X A  -  X B*B' X  =:  R(X)
%
%  Input:
%    Y0      - initial starting guess s.t.  A - B*B'*Y0'*Y0 is stable.
%              Note: this is not checked - if Y0 is not stabilizing, then
%              the iteration may fail to converge or converge to a
%              non-stabilizing solution! 
%    A, B, C - matrices from (CARE).
%    tol     - stopping criterion, i.e., the iteration is stopped if 
%
%           norm(R(X),'fro')  
%		------------------------   <=  tol . 
%		 max(1.0,norm(Y'*Y,'fro')) 
%
%              
%    maxsteps - maximum number of iteration steps, default = 50.
%
%  Output:
%    Y      : approximate factor solution of CARE so that X = Y'*Y.
%    record : record(k,1) = norm(R(X),'fro')  (k-th iteration),
%             record(k,2) = norm(S,'fro')     (k-th iteration).
% REFERENCE:
%
%   
%
% Peter Benner,  2005-12-23.

error(nargchk(4,6,nargin))
n = size(A,1);
m = size(B,2);
p = size(C,1);

%check matrix sizes
if size(A,2) ~= n,  
    error('A must be square.'),  
end
if size(B,1) ~= n,  
    error('B must have the same number of rows as A.'),  
end
if size(C,2) ~= n,  
    error('C must have the same number of columns as A.'),  
end

if nargin < 6,  maxsteps = 50;  end
if (nargin < 5),  
  tol = sqrt(eps*n);
else
    if tol < sqrt(eps*n)
        tol = sqrt(eps*n);
        warning('Error tolerance too small, may not be achieved!')
    end
end

% Initialization
iter = 0;
if isempty(Y0),
    Y = zeros(1,n);
else
    if size(Y0,2) ~= n,  
        error('Y0 must have the same number of rows as A.'),  
    else
        Y = Y0;
    end
end
YA    = Y*A;
YB    = Y*B;
nres  = norm(C'*C + YA'*Y + Y'*YA - YB*YB','fro');     
Xnorm = norm(Y*Y','fro');
Err = nres/max(1,Xnorm);
onemore     = 0;
convergence = Err <= tol;

% Newton iteration
while (iter < maxsteps) & ((~convergence) | (convergence & (onemore < 2))),
  % Here one may employ RRQR to compress W.
  W        = [C; YB'*Y]; 
  Y        = lyap_sgn_fac(A - B*(YB)'*Y,W); 
  YA       = Y*A;
  YB       = Y*B;
  nres     = norm(C'*C + YA'*Y + Y'*YA - (Y'*YB)*(YB'*Y),'fro');     
  Xnorm    = norm(Y*Y','fro');
  iter = iter + 1;
% Uncomment next line for verbose mode.
 disp(sprintf('||R(X_%i)||/||X|| = %d', iter, nres/Xnorm))
  Err = nres/max(1,Xnorm);
  convergence = Err <= tol;
  if convergence,  onemore = onemore + 1;  end
end

if (iter == maxsteps) & (nres/max(1,Xnorm) > tol),
  disp(sprintf('CARE_NWT_FAC: no convergence in %d iterations', maxsteps))
end