function [S,R,iter] = lyapdl_sgn_fac(A,B,C);
%LYAPDL_SGN_FAC
%
% Solve the stable dual Lyapunov equations
%
% (*)  A X + X A' + B B' = 0,   A' Y + Y A + C' C = 0
%
% for a full-rank factors of X,Y via the sign function iteration.
%
% Input:  A - a square, n x n - matrix.
%         B - an n x m - matrix
%         C - a p x n - matrix
%
% Output: S,R - numerical full rank factors of X and Y.
%         iter (optionally) - number of sign iterations required. 
%
% REFERENCE:
%
%   P. BENNER, E.S. QUINTANA-ORTI. Model reduction based on spectral 
%   projection methods.  In: P. Benner, V.L. Mehrmann, D.C. Sorensen
%   (eds.), "Dimension Redution of Large-Scale Systems", vol. 45 of
%   Lecture Notes in Computational Science and Engineering, pp. 5-48,
%   Springer-Verlag, Berlin/Heidelberg, 2005.
%
% Peter Benner, 2004-2005.

n = size(A,1);

iter= 0;
tol = 10*sqrt(n*eps);
maxstep = 50;
S    = B';
R    = C;
E    = eye(n);
Err  = norm(A + E,'fro');
onemore     = 0;
convergence = Err <= tol;

while (iter < maxstep) & ((~convergence) | (convergence & (onemore < 3))),
  [L,U,P] = lu(A);
  Y = L\P;
  Y = U\Y;
  if Err > 0.1,
    d = sqrt(norm(A,'fro')/norm(Y,'fro'));
  else
    d = 1;
  end
  A = (A/d + d*Y)/2;
  S = [S; d*S*Y']/sqrt(2*d);
  %should be replace by true RRQR!
  [U,T,P] = qr(S);
  [p,dummy] = find(P'==1);
  r = rank(T,tol);
  S = T(1:r,p);
  R = [R; d*R*Y]/sqrt(2*d);  
  [U,T,P] = qr(R);
  [p,dummy] = find(P'==1);
  r = rank(T,tol);
  R = T(1:r,p);
  Err  = norm(A + E,'fro');
  iter = iter + 1;
  %% comment out for non-verbose mode %%
  %%disp(sprintf('Step %i, conv. crit. = %d', iter, Err))
  convergence = Err <= tol;
  if convergence,  onemore = onemore + 1;  end
end
S = S'/sqrt(2);
R = R/sqrt(2);
if (iter == maxstep & norm(A + E,'fro') > tol),
  disp(sprintf('LYAPDL_SGN_FAC: No convergence in %d iterations.', maxstep))
end