function [Ar,Br,Cr,Dr,Er] = mt(A,B,C,D,E,alpha)
% MT
% Compute a reduced-order model of a given linear, continuous-time system
% using modal truncation based on a spectral projection approach.
% 
% INPUT ARGUMENTS:
%    A  - an n x n - matrix.  Note that A or (A,E) if E is nonempty) is
%         assumed to be stable - this is not checked in the code!  An
%         unstable A-matrix will usually cause the sign function-based
%         Lyapunov solver to exit with an error due to failure to converge.
%    B  - an n x m - matrix.
%    C  - an p x n - matrix.
%    D  - an p x m - matrix (D is allowed to be empty).
%    E  - a nonsingular n x n - matrix (E is allowed to be empty, then E is 
%         taken as identity matrix of order n).
%   alpha - determines the modes to be preserved as follows: if EV is an 
%           eigenvalue of the A-matrix of the system, then it is presered
%           in the reduced-order model iff Real(EV) > -ALPHA.
%
% OUTPUT ARGUMENTS:
%   rom    - an SS object containing the reduced-order model. 
%
% REFERENCE:
%   P. BENNER, E.S. QUINTANA-ORTI. Model reduction based on spectral 
%   projection methods.  In: P. Benner, V.L. Mehrmann, D.C. Sorensen
%   (eds.), "Dimension Redution of large-scale systems", vol. xx of
%   Lecture Notes in Computational Science and Engineering, pp. yy-zz,
%   Springer-Verlag, Berlin/Heidelberg, 2005.
%
% Peter Benner,  2004-2005.

error(nargchk(3, 6, nargin))
n = size(A,1);
m = size(B,2);
p = size(C,1);
if nargin <= 3,
    D = [];
    E = [];
end

%check matrix sizes
if size(A,2) ~= n,  
    error('A must be square.'),  
end
if size(B,1) ~= n,  
    error('B must have the same number of rows as A.'),  
end
if size(C,2) ~= n,  
    error('C must have the same number of columns as A.'),  
end
if ~isempty(E),
    if (size(E,1)~=n | size(E,2)~=n)
        error('E must have the same size as A.')
    else
        A = E\A;
        B = E\B;
    end
end

if (nargin < 5),
    alpha = 1;
end

n = size(A,1);

% Block-triangularization of A and update of B,C
S = signm(A+alpha*eye(n),2);
[Q,R,P] = qr(eye(n)+S);
r = (n + round(trace(S)))/2;
A = Q'*A*Q;
B = Q'*B;
C = C*Q;

% Compute block-diagonalizing transformation
Y = sylv_sgn(-A(1:r,1:r)-alpha*eye(r),(A(r+1:n,r+1:n)+alpha*eye(n-r)),...
    A(1:r,r+1:n));

% Compute reduced-order model
Ar = A(1:r,1:r);
Br = B(1:r,:) - Y*B(r+1:n,:);
Cr = C(:,1:r);
Dr = D;
if isempty(E),
    Er = [];
else
    if nargout > 4,
        E = eye(n);
    end
end