function [Ar,Br,Cr,r,hksv] = srrom(A,B,C,S,R,tolselect,tau);
%SRROM
%
% Compute a reduced-order model using the SR variant of balanced
% truncation from given factors of system Gramians.
%
% INPUT ARGUMENTS:
%    A      - an n x n - matrix.
%    B      - an n x m - matrix.
%    C      - an p x n - matrix.
%    S      - an n_S x n - matrix
%    R      - an n_R x n - matrix.
% tolselect - If tolselect == 1, then the order of the reduced-order model 
%             is chosen based on the tolerance provided in tau. 
%             Else, the order of the reduced-order model 
%             is chosen based on the number tau.
% tau       - Either a tolerance for the approximation error or the size of the 
%             reduced-order model, depending on tolselect.    
%
% OUTPUT ARGUMENTS:
%   Ar,Br,Cr - system matrices of the reduced-order model.  If D/E is 
%                    empty on input, Dr/Er are empty on output. 
%   r        - the order of the reduced-order model.
%   hksv     - an optional output argument containing the computed Hankel
%              singular values.
%
%
% Peter Benner, 2006-02-10.

%%Check input arguments - it is assumed that the sizes of all input
%%matrices are checked for consistency in the calling function.
error(nargchk(7, 7, nargin))

%% Compute Hankel singular values.
[U,Sigma,V]=svd(S'*R',0);
hksv = diag(Sigma);

%% Select order of reduced-order model
s = min(size(S,2),size(R,1));
if tolselect,
    tol    = tau;
    abserr = 0;
    r      = s+1;
    while abserr < tol/2,
        r      = r-1;
        abserr = abserr + hksv(r);
    end
else
    r = tau;
    if r > s,
        warning('Desired size of reduced-order model too large.')
        disp(sprintf('         Set r = %i', s'))
        r = s;
    end
end

%% Compute reduced-order model
S1  = diag(1./sqrt(hksv(1:r)));
Tl  = S1*V(:,1:r)'*R;
Tr  = S*U(:,1:r)*S1;
Ar  = Tl*A*Tr;
Br  = Tl*B;
Cr  = C*Tr;