function [X] = signsylv(A,B,C,maxit);
%
% function [X] = signsylv(A,B,C,maxit);
%
%
% Solve the "stable" Sylvester equation 
%
% (*)    A X + X B = C 
%   
% via the Newton iteration for the sign function.
%
% INPUT: A, B, C, the coefficient matrices in (*).
%        It is required that A, B are square and nonsingular, 
%        size(C) == [size(A,1), size(B,2)], 
%        and that eig(A) and eig(B) are in the open left half
%        plane. NOTE: the assumption on the spectra is not checked! 
%
% OUTPUT: X, the solution of (*). 
%
% Peter Benner, 2005-02-04  

n = size(A,1);
m = size(B,1);
if (size(C) ~= [n,m]),  error('C has wrong size.'),  end
if nargin < 4,  maxit = 50;  end
Aold = A;
Bold = B;
Cold = C;
%
iter    = 0;
ittol = 10*n*sqrt(eps); 
%
X    = C;
Anrm = norm(A,'fro');
Bnrm = norm(B,'fro');
%
Err         = max([norm(A+eye(n),'fro')/Anrm,norm(B+eye(m),'fro')/Bnrm]);
onemore     = 0;
convergence = Err <= ittol;

while (iter < maxit) & ((~convergence) | (convergence & (onemore < 3))),   
  [L,U,P] = lu(A);
  YA = (U\(L\P));
  [L,U,P] = lu(B);
  YB   = (U\(L\P));
  W  = YA*X*YB;
  en = sqrt( norm(A,'fro')^2 + norm(X,'fro')^2 + norm(B,'fro')^2 );
  de = sqrt( norm(YA,'fro')^2 + norm(W,'fro')^2 + norm(YB,'fro')^2 );  
  cs = sqrt(en/de);
  A    = (A/cs + cs*YA)/2;
  B    = (B/cs + cs*YB)/2;
  X    = (X/cs + cs*W)/2;
  Err  = max([norm(A+eye(n),'fro')/Anrm,norm(B+eye(m),'fro')/Bnrm]);
  iter = iter + 1;
  %% comment out for non-verbose mode %%
  %%disp(sprintf('Newton step %i, conv. crit. = %d', iter, Err))
  convergence = Err <= ittol;
  if convergence,  onemore = onemore + 1;  end
end
%
X = -X/2;

if (iter == maxit & Err > ittol),
  disp(sprintf('SIGNSYLV: No convergence in %d iterations.', maxit))
end
